package be.iminds.ilabt.jfed.experimenter_gui.slice;

import be.iminds.ilabt.jfed.experimenter_gui.dialogs.JFedBrowserFactory;
import be.iminds.ilabt.jfed.experimenter_gui.util.BrowserUtil;
import javafx.scene.control.*;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;

import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;

/**
 * OpenNodeBrowserDialog
 */
public class OpenNodeBrowserDialog extends Dialog<String> {
    private final BrowserUtil browserUtil;
    private final JFedBrowserFactory jFedBrowserFactory;

    private final RadioButton internalRadioButton;

    public OpenNodeBrowserDialog(BrowserUtil browserUtil,
                                 JFedBrowserFactory jFedBrowserFactory,
                                 Collection<String> possibleHostNames) {
        super();
        this.browserUtil = browserUtil;
        this.jFedBrowserFactory = jFedBrowserFactory;

        setTitle("Open browser to node");
        setHeaderText("Open browser to node");

        HBox urlLine = new HBox(5);
        ComboBox<String> protocol = new ComboBox<String>();
        protocol.setPrefWidth(110);
        protocol.getItems().add("http://");
        protocol.getItems().add("https://");

        protocol.getSelectionModel().selectFirst();

        ComboBox<String> hostname = new ComboBox<>();
        hostname.setPrefWidth(400);
        hostname.getItems().setAll(possibleHostNames);

        hostname.getSelectionModel().selectFirst();

        Spinner<Integer> portSpinner = new Spinner<>(0, 10240, 80);
        portSpinner.setEditable(true);
        portSpinner.setPrefWidth(100);

        protocol.getSelectionModel().selectedItemProperty().addListener(
                (prop, oldValue, newValue) -> {
                    if (Objects.equals(oldValue, newValue))
                        return;
                    if (portSpinner.getValue() == 80 && Objects.equals(newValue, "https://"))
                        portSpinner.getValueFactory().setValue(443);
                    if (portSpinner.getValue() == 443 && Objects.equals(newValue, "http://"))
                        portSpinner.getValueFactory().setValue(80);
                }
        );

        urlLine.getChildren().addAll(
                new javafx.scene.control.Label("Select the URL:"),
                protocol,
                hostname,
                new javafx.scene.control.Label(":"),
                portSpinner);

        RadioButton externalRadioButton = new RadioButton("Use System Browser");
        internalRadioButton = new RadioButton("Use Internal jFed Browser");
        ToggleGroup rbGroup = new ToggleGroup();
        externalRadioButton.setToggleGroup(rbGroup);
        internalRadioButton.setToggleGroup(rbGroup);
        externalRadioButton.setSelected(true);

        VBox content = new VBox();
        content.getChildren().addAll(urlLine, externalRadioButton, internalRadioButton);

        getDialogPane().setContent(content);
        getDialogPane().getButtonTypes().setAll(ButtonType.OK, ButtonType.CANCEL);

        setResultConverter(param -> {
            if (param == ButtonType.OK) {
                return protocol.getSelectionModel().getSelectedItem() + hostname.getSelectionModel().getSelectedItem() + ":" + portSpinner.getValue().toString();
            } else
                return null;
        });

    }

    private void openBrowser(String url) {
        if (internalRadioButton.isSelected()) {
            jFedBrowserFactory.showDialog(url);
        } else {
            browserUtil.openUrlInBrowser(url);
        }
    }

    public void doDialog() {
        showAndWait().ifPresent(this::openBrowser);
    }

}
